#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# SPDX-License-Identifier: Apache-2.0
"""SDC Element module."""
from typing import Any, Dict, List
from operator import attrgetter
from abc import ABC, abstractmethod

from requests import Response

from onapsdk.configuration import settings
from onapsdk.exceptions import APIError, RequestError
from onapsdk.onap_service import OnapService
import onapsdk.constants as const
from onapsdk.utils.jinja import jinja_env


class SDC(OnapService, ABC):
    """Mother Class of all SDC elements."""

    server: str = "SDC"
    base_front_url = settings.SDC_FE_URL
    base_back_url = settings.SDC_BE_URL

    def __init__(self, name: str = None) -> None:
        """Initialize SDC."""
        super().__init__()
        self.name: str = name

    def __eq__(self, other: Any) -> bool:
        """
        Check equality for SDC and children.

        Args:
            other: another object

        Returns:
            bool: True if same object, False if not

        """
        if isinstance(other, type(self)):
            return self.name == other.name
        return False

    @classmethod
    @abstractmethod
    def _get_all_url(cls) -> str:
        """
        Get URL for all elements in SDC.

        Raises:
            NotImplementedError: this is an abstract method.

        """

    @classmethod
    @abstractmethod
    def _get_objects_list(cls,
                          result: List[Dict[str, Any]]) -> List['SdcResource']:
        """
        Import objects created in SDC.

        Args:
            result (Dict[str, Any]): the result returned by SDC in a Dict

        Raises:
            NotImplementedError: this is an abstract method.

        """

    @classmethod
    @abstractmethod
    def _base_url(cls) -> str:
        """
        Give back the base url of Sdc.

        Raises:
            NotImplementedError: this is an abstract method.

        """

    @classmethod
    @abstractmethod
    def _base_create_url(cls) -> str:
        """
        Give back the base url of Sdc.

        Raises:
            NotImplementedError: this is an abstract method.

        """

    @abstractmethod
    def _copy_object(self, obj: 'SDC') -> None:
        """
        Copy relevant properties from object.

        Args:
            obj (Sdc): the object to "copy"

        Raises:
            NotImplementedError: this is an abstract method.

        """

    @classmethod
    @abstractmethod
    def import_from_sdc(cls, values: Dict[str, Any]) -> 'SDC':
        """
        Import Sdc object from SDC.

        Args:
            values (Dict[str, Any]): dict to parse returned from SDC.

        Raises:
            NotImplementedError: this is an abstract method.

        """

    @classmethod
    def get_all(cls, **kwargs) -> List['SDC']:
        """
        Get the objects list created in SDC.

        Returns:
            the list of the objects

        """
        cls._logger.info("retrieving all objects of type %s from SDC",
                         cls.__name__)
        url = cls._get_all_url()
        objects = []

        try:
            result = \
                cls.send_message_json('GET', "get {}s".format(cls.__name__),
                                      url, **kwargs)

            for obj_info in cls._get_objects_list(result):
                objects.append(cls.import_from_sdc(obj_info))

        except APIError as exc:
            cls._logger.debug("Couldn't get %s: %s", cls.__name__, exc)
        except KeyError as exc:
            cls._logger.debug("Invalid result dictionary: %s", exc)

        cls._logger.debug("number of %s returned: %s", cls.__name__,
                          len(objects))
        return objects

    def exists(self) -> bool:
        """
        Check if object already exists in SDC and update infos.

        Returns:
            True if exists, False either

        """
        self._logger.debug("check if %s %s exists in SDC",
                           type(self).__name__, self.name)
        objects = self.get_all()

        self._logger.debug("filtering objects of all versions to be %s",
                           self.name)
        relevant_objects = list(filter(lambda obj: obj == self, objects))

        if not relevant_objects:

            self._logger.info("%s %s doesn't exist in SDC",
                              type(self).__name__, self.name)
            return False

        if hasattr(self, 'version_filter') and self.version_filter is not None: # pylint: disable=no-member

            self._logger.debug("filtering %s objects by version %s",
                               self.name, self.version_filter) # pylint: disable=no-member

            all_versioned = filter(
                lambda obj: obj.version == self.version_filter, relevant_objects) # pylint: disable=no-member

            try:
                versioned_object = next(all_versioned)
            except StopIteration:
                self._logger.info("Version %s of %s %s, doesn't exist in SDC",
                                  self.version_filter, type(self).__name__,  # pylint: disable=no-member
                                  self.name)
                return False

        else:
            versioned_object = max(relevant_objects, key=attrgetter('version'))

        self._logger.info("%s found, updating information", type(self).__name__)
        self._copy_object(versioned_object)
        return True


class SdcOnboardable(SDC, ABC):
    """Base class for onboardable SDC resources (Vendors, Services, ...)."""

    ACTION_TEMPLATE: str
    ACTION_METHOD: str

    def __init__(self, name: str = None) -> None:
        """Initialize the object."""
        super().__init__(name)
        self._identifier: str = None
        self._status: str = None
        self._version: str = None

    @property
    def identifier(self) -> str:
        """Return and lazy load the identifier."""
        if not self._identifier:
            self.load()
        return self._identifier

    @property
    def status(self) -> str:
        """Return and lazy load the status."""
        if self.created() and not self._status:
            self.load()
        return self._status

    @property
    def version(self) -> str:
        """Return and lazy load the version."""
        if self.created() and not self._version:
            self.load()
        return self._version

    @identifier.setter
    def identifier(self, value: str) -> None:
        """Set value for identifier."""
        self._identifier = value

    @status.setter
    def status(self, status: str) -> None:
        """Return and lazy load the status."""
        self._status = status

    @version.setter
    def version(self, version: str) -> None:
        """Return and lazy load the status."""
        self._version = version

    def created(self) -> bool:
        """Determine if SDC is created."""
        if self.name and not self._identifier:
            return self.exists()
        return bool(self._identifier)

    def submit(self) -> None:
        """Submit the SDC object in order to enable it."""
        self._logger.info("attempting to certify/sumbit %s %s in SDC",
                          type(self).__name__, self.name)
        if self.status != const.CERTIFIED and self.created():
            self._really_submit()
        elif self.status == const.CERTIFIED:
            self._logger.warning("%s %s in SDC is already submitted/certified",
                                 type(self).__name__, self.name)
        elif not self.created():
            self._logger.warning("%s %s in SDC is not created",
                                 type(self).__name__, self.name)

    def _create(self, template_name: str, **kwargs) -> None:
        """Create the object in SDC if not already existing."""
        self._logger.info("attempting to create %s %s in SDC",
                          type(self).__name__, self.name)
        if not self.exists():
            url = "{}/{}".format(self._base_create_url(), self._sdc_path())
            template = jinja_env().get_template(template_name)
            data = template.render(**kwargs)
            try:
                create_result = self.send_message_json('POST',
                                                       "create {}".format(
                                                           type(self).__name__),
                                                       url,
                                                       data=data)
            except RequestError as exc:
                self._logger.error(
                    "an error occured during creation of %s %s in SDC",
                    type(self).__name__, self.name)
                raise exc
            else:
                self._logger.info("%s %s is created in SDC",
                                  type(self).__name__, self.name)
                self._status = const.DRAFT
                self.identifier = self._get_identifier_from_sdc(create_result)
                self._version = self._get_version_from_sdc(create_result)
                self.update_informations_from_sdc_creation(create_result)

        else:
            self._logger.warning("%s %s is already created in SDC",
                                 type(self).__name__, self.name)

    def _action_to_sdc(self, action: str, action_type: str = None,
                       **kwargs) -> Response:
        """
        Really do an action in the SDC.

        Args:
            action (str): the action to perform
            action_type (str, optional): the type of action
            headers (Dict[str, str], optional): headers to use if any

        Returns:
            Response: the response

        """
        subpath = self._generate_action_subpath(action)
        url = self._action_url(self._base_create_url(),
                               subpath,
                               self._version_path(),
                               action_type=action_type)
        template = jinja_env().get_template(self.ACTION_TEMPLATE)
        data = template.render(action=action, const=const)

        return self.send_message(self.ACTION_METHOD,
                                 "{} {}".format(action,
                                                type(self).__name__),
                                 url,
                                 data=data,
                                 **kwargs)

    @abstractmethod
    def update_informations_from_sdc(self, details: Dict[str, Any]) -> None:
        """

        Update instance with details from SDC.

        Args:
            details ([type]): [description]

        """
    @abstractmethod
    def update_informations_from_sdc_creation(self,
                                              details: Dict[str, Any]) -> None:
        """

        Update instance with details from SDC after creation.

        Args:
            details ([type]): the details from SDC

        """

    @abstractmethod
    def load(self) -> None:
        """
        Load Object information from SDC.

        Raises:
            NotImplementedError: this is an abstract method.

        """

    @abstractmethod
    def _get_version_from_sdc(self, sdc_infos: Dict[str, Any]) -> str:
        """
        Get version from SDC results.

        Args:
            sdc_infos (Dict[str, Any]): the result dict from SDC

        Raises:
            NotImplementedError: this is an abstract method.

        """
    @abstractmethod
    def _get_identifier_from_sdc(self, sdc_infos: Dict[str, Any]) -> str:
        """
        Get identifier from SDC results.

        Args:
            sdc_infos (Dict[str, Any]): the result dict from SDC

        Raises:
            NotImplementedError: this is an abstract method.

        """
    @abstractmethod
    def _generate_action_subpath(self, action: str) -> str:
        """

        Generate subpath part of SDC action url.

        Args:
            action (str): the action that will be done

        Raises:
            NotImplementedError: this is an abstract method.

        """
    @abstractmethod
    def _version_path(self) -> str:
        """
        Give the end of the path for a version.

        Raises:
            NotImplementedError: this is an abstract method.

        """
    @abstractmethod
    def _really_submit(self) -> None:
        """Really submit the SDC Vf in order to enable it."""
    @staticmethod
    @abstractmethod
    def _action_url(base: str,
                    subpath: str,
                    version_path: str,
                    action_type: str = None) -> str:
        """
        Generate action URL for SDC.

        Raises:
            NotImplementedError: this is an abstract method.

        """
    @classmethod
    @abstractmethod
    def _sdc_path(cls) -> None:
        """Give back the end of SDC path."""

    @abstractmethod
    def onboard(self) -> None:
        """Onboard resource.

        Onboarding is a full stack of actions which needs to be done to
            make SDC resource ready to use. It depends on the type of object
            but most of them needs to be created and submitted.
        """



"""Base instance module."""

from abc import ABC, abstractmethod

from ..aai_element import AaiElement


class Instance(AaiElement, ABC):
    """Abstract instance class."""

    def __init__(self,  # pylint: disable=too-many-arguments
                 resource_version: str = None,
                 model_invariant_id: str = None,
                 model_version_id: str = None) -> None:
        """Instance initialization.

        Args:
            resource_version (str, optional): Used for optimistic concurrency.
                Must be empty on create, valid on update and delete. Defaults to None.
            model_invariant_id (str, optional): The ASDC model id for this resource or
                service model. Defaults to None.
            model_version_id (str, optional): The ASDC model version for this resource or
                service model. Defaults to None.
        """
        super().__init__()
        self.resource_version: str = resource_version
        self.model_invariant_id: str = model_invariant_id
        self.model_version_id: str = model_version_id

    @abstractmethod
    def delete(self) -> "DeletionRequest":
        """Create instance deletion request.

        Send request to delete instance

        Returns:
            DeletionRequest: Deletion request

        """



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# SPDX-License-Identifier: Apache-2.0
"""Deletion module."""
from abc import ABC

from onapsdk.onap_service import OnapService
from onapsdk.utils.headers_creator import headers_so_creator
from onapsdk.utils.jinja import jinja_env

from onapsdk.so.so_element import OrchestrationRequest


class DeletionRequest(OrchestrationRequest, ABC):
    """Deletion request base class."""

    @classmethod
    def send_request(cls, instance: "AaiElement") -> "Deletion":
        """Abstract method to send instance deletion request.

        Raises:
            NotImplementedError: Needs to be implemented in inheriting classes

        """
        raise NotImplementedError


class VfModuleDeletionRequest(DeletionRequest):  # pytest: disable=too-many-ancestors
    """VF module deletion class."""

    @classmethod
    def send_request(cls,
                     instance: "VfModuleInstance") -> "VfModuleDeletion":
        """Send request to SO to delete VNF instance.

        Args:
            vnf_instance (VnfInstance): VNF instance to delete

        Returns:
            VnfDeletionRequest: Deletion request object

        """
        cls._logger.debug("VF module %s deletion request", instance.vf_module_id)
        response = cls.send_message_json("DELETE",
                                         (f"Create {instance.vf_module_id} VF module"
                                          "deletion request"),
                                         (f"{cls.base_url}/onap/so/infra/"
                                          f"serviceInstantiation/{cls.api_version}/"
                                          "serviceInstances/"
                                          f"{instance.vnf_instance.service_instance.instance_id}/"
                                          f"vnfs/{instance.vnf_instance.vnf_id}/"
                                          f"vfModules/{instance.vf_module_id}"),
                                         data=jinja_env().
                                         get_template("deletion_vf_module.json.j2").
                                         render(vf_module_instance=instance),
                                         headers=headers_so_creator(OnapService.headers))
        return cls(request_id=response["requestReferences"]["requestId"])


class VnfDeletionRequest(DeletionRequest):  # pytest: disable=too-many-ancestors
    """VNF deletion class."""

    @classmethod
    def send_request(cls,
                     instance: "VnfInstance") -> "VnfDeletionRequest":
        """Send request to SO to delete VNF instance.

        Args:
            instance (VnfInstance): VNF instance to delete

        Returns:
            VnfDeletionRequest: Deletion request object

        """
        cls._logger.debug("VNF %s deletion request", instance.vnf_id)
        response = cls.send_message_json("DELETE",
                                         f"Create {instance.vnf_id} VNF deletion request",
                                         (f"{cls.base_url}/onap/so/infra/"
                                          f"serviceInstantiation/{cls.api_version}/"
                                          "serviceInstances/"
                                          f"{instance.service_instance.instance_id}/"
                                          f"vnfs/{instance.vnf_id}"),
                                         data=jinja_env().
                                         get_template("deletion_vnf.json.j2").
                                         render(vnf_instance=instance),
                                         headers=headers_so_creator(OnapService.headers))
        return cls(request_id=response["requestReferences"]["requestId"])


class ServiceDeletionRequest(DeletionRequest):  # pytest: disable=too-many-ancestors
    """Service deletion request class."""

    @classmethod
    def send_request(cls,
                     instance: "ServiceInstance") -> "ServiceDeletionRequest":
        """Send request to SO to delete service instance.

        Args:
            instance (ServiceInstance): service instance to delete

        Returns:
            ServiceDeletionRequest: Deletion request object

        """
        cls._logger.debug("Service %s deletion request", instance.instance_id)
        response = cls.send_message_json("DELETE",
                                         f"Create {instance.instance_id} Service deletion request",
                                         (f"{cls.base_url}/onap/so/infra/"
                                          f"serviceInstantiation/{cls.api_version}/"
                                          f"serviceInstances/{instance.instance_id}"),
                                         data=jinja_env().
                                         get_template("deletion_service.json.j2").
                                         render(service_instance=instance),
                                         headers=headers_so_creator(OnapService.headers))
        return cls(request_id=response["requestReferences"]["requestId"])


class NetworkDeletionRequest(DeletionRequest):  # pylint: disable=too-many-ancestors
    """Network deletion request class."""

    @classmethod
    def send_request(cls,
                     instance: "NetworkInstance") -> "VnfDeletionRequest":
        """Send request to SO to delete Network instance.

        Args:
            instance (NetworkInstance): Network instance to delete

        Returns:
            NetworkDeletionRequest: Deletion request object

        """
        cls._logger.debug("Network %s deletion request", instance.network_id)
        response = cls.send_message_json("DELETE",
                                         f"Create {instance.network_id} Network deletion request",
                                         (f"{cls.base_url}/onap/so/infra/"
                                          f"serviceInstantiation/{cls.api_version}/"
                                          "serviceInstances/"
                                          f"{instance.service_instance.instance_id}/"
                                          f"networks/{instance.network_id}"),
                                         data=jinja_env().
                                         get_template("deletion_network.json.j2").
                                         render(network_instance=instance),
                                         headers=headers_so_creator(OnapService.headers))
        return cls(request_id=response["requestReferences"]["requestId"])



"""CDS Blueprint Models module."""

from typing import Iterator
from onapsdk.exceptions import ResourceNotFound  # for custom exceptions

from .blueprint import Blueprint
from .cds_element import CdsElement


class BlueprintModel(CdsElement):  # pylint: disable=too-many-instance-attributes
    """Blueprint Model class.

    Represents blueprint models in CDS
    """

    def __init__(self,  # pylint: disable=too-many-arguments
                 blueprint_model_id: str,
                 artifact_uuid: str = None,
                 artifact_type: str = None,
                 artifact_version: str = None,
                 artifact_description: str = None,
                 internal_version: str = None,
                 created_date: str = None,
                 artifact_name: str = None,
                 published: str = 'N',
                 updated_by: str = None,
                 tags: str = None):
        """Blueprint Model initialization.

        Args:
            blueprint_model_id (str): Blueprint model identifier
            artifact_uuid (str): Blueprint model uuid
            artifact_type (str): Blueprint artifact type
            artifact_version (str): Blueprint model version
            artifact_description (str): Blueprint model description
            internal_version (str): Blueprint model internal version
            created_date (str): Blueprint model created date
            artifact_name (str): Blueprint model name
            published (str): Blueprint model publish status - 'N' or 'Y'
            updated_by (str): Blueprint model author
            tags (str): Blueprint model tags

        """
        super().__init__()
        self.blueprint_model_id = blueprint_model_id
        self.artifact_uuid = artifact_uuid
        self.artifact_type = artifact_type
        self.artifact_version = artifact_version
        self.artifact_description = artifact_description
        self.internal_version = internal_version
        self.created_date = created_date
        self.artifact_name = artifact_name
        self.published = published
        self.updated_by = updated_by
        self.tags = tags

    def __repr__(self) -> str:
        """Representation of object.

        Returns:
           str: Object's string representation

        """
        return (f"BlueprintModel(artifact_name='{self.artifact_name}', "
                f"blueprint_model_id='{self.blueprint_model_id}')")

    @classmethod
    def get_by_id(cls, blueprint_model_id: str) -> "BlueprintModel":
        """Retrieve blueprint model with provided ID.

        Args: blueprint_model_id (str):

        Returns:
            BlueprintModel: Blueprint model object

        Raises:
            ResourceNotFound: Blueprint model with provided ID doesn't exist

        """
        try:
            blueprint_model = cls.send_message_json(
                "GET",
                "Retrieve blueprint",
                f"{cls._url}/api/v1/blueprint-model/{blueprint_model_id}",
                auth=cls.auth)

            return cls(
                blueprint_model_id=blueprint_model["blueprintModel"]['id'],
                artifact_uuid=blueprint_model["blueprintModel"]['artifactUUId'],
                artifact_type=blueprint_model["blueprintModel"]['artifactType'],
                artifact_version=blueprint_model["blueprintModel"]['artifactVersion'],
                internal_version=blueprint_model["blueprintModel"]['internalVersion'],
                created_date=blueprint_model["blueprintModel"]['createdDate'],
                artifact_name=blueprint_model["blueprintModel"]['artifactName'],
                published=blueprint_model["blueprintModel"]['published'],
                updated_by=blueprint_model["blueprintModel"]['updatedBy'],
                tags=blueprint_model["blueprintModel"]['tags']
            )

        except ResourceNotFound:
            raise ResourceNotFound(f"BlueprintModel blueprint_model_id='{blueprint_model_id}"
                                   f" not found")

    @classmethod
    def get_by_name_and_version(cls, blueprint_name: str,
                                blueprint_version: str) -> "BlueprintModel":
        """Retrieve blueprint model with provided name and version.

        Args:
            blueprint_name (str): Blueprint model name
            blueprint_version (str): Blueprint model version

        Returns:
            BlueprintModel: Blueprint model object

        Raises:
            ResourceNotFound: Blueprint model with provided name and version doesn't exist

        """
        try:
            blueprint_model = cls.send_message_json(
                "GET",
                "Retrieve blueprint",
                f"{cls._url}/api/v1/blueprint-model/by-name/{blueprint_name}"
                f"/version/{blueprint_version}",
                auth=cls.auth)

            return cls(
                blueprint_model_id=blueprint_model["blueprintModel"]['id'],
                artifact_uuid=blueprint_model["blueprintModel"]['artifactUUId'],
                artifact_type=blueprint_model["blueprintModel"]['artifactType'],
                artifact_version=blueprint_model["blueprintModel"]['artifactVersion'],
                internal_version=blueprint_model["blueprintModel"]['internalVersion'],
                created_date=blueprint_model["blueprintModel"]['createdDate'],
                artifact_name=blueprint_model["blueprintModel"]['artifactName'],
                published=blueprint_model["blueprintModel"]['published'],
                updated_by=blueprint_model["blueprintModel"]['updatedBy'],
                tags=blueprint_model["blueprintModel"]['tags']
            )

        except ResourceNotFound:
            raise ResourceNotFound(f"BlueprintModel blueprint_name='{blueprint_name}"
                                   f" and blueprint_version='{blueprint_version}' not found")

    @classmethod
    def get_all(cls) -> Iterator["BlueprintModel"]:
        """Get all blueprint models.

        Yields:
            BlueprintModel: BlueprintModel object.

        """
        for blueprint_model in cls.send_message_json(
                "GET",
                "Retrieve all blueprints",
                f"{cls._url}/api/v1/blueprint-model",
                auth=cls.auth):

            yield cls(
                blueprint_model_id=blueprint_model["blueprintModel"]['id'],
                artifact_uuid=blueprint_model["blueprintModel"]['artifactUUId'],
                artifact_type=blueprint_model["blueprintModel"]['artifactType'],
                artifact_version=blueprint_model["blueprintModel"]['artifactVersion'],
                internal_version=blueprint_model["blueprintModel"]['internalVersion'],
                created_date=blueprint_model["blueprintModel"]['createdDate'],
                artifact_name=blueprint_model["blueprintModel"]['artifactName'],
                published=blueprint_model["blueprintModel"]['published'],
                updated_by=blueprint_model["blueprintModel"]['updatedBy'],
                tags=blueprint_model["blueprintModel"]['tags']
            )

    def get_blueprint(self) -> Blueprint:
        """Get Blueprint object for selected blueprint model.

        Returns:
            Blueprint: Blueprint object

        """
        cba_package = self.send_message(
            "GET",
            "Retrieve selected blueprint object",
            f"{self._url}/api/v1/blueprint-model/download/{self.blueprint_model_id}",
            auth=self.auth)

        return Blueprint(cba_file_bytes=cba_package.content)

    def save(self, dst_file_path: str):
        """Save blueprint model to file.

        Args:
            dst_file_path (str): Path of file where blueprint is going to be saved
        """
        cba_package = self.send_message(
            "GET",
            "Retrieve and save selected blueprint",
            f"{self._url}/api/v1/blueprint-model/download/{self.blueprint_model_id}",
            auth=self.auth)

        with open(dst_file_path, "wb") as content:
            for chunk in cba_package.iter_content(chunk_size=128):
                content.write(chunk)

    def delete(self):
        """Delete blueprint model."""
        self.send_message(
            "DELETE",
            "Delete blueprint",
            f"{self._url}/api/v1/blueprint-model/{self.blueprint_model_id}",
            auth=self.auth)



# SPDX-License-Identifier: Apache-2.0
"""Base CDS module."""
from abc import ABC

from onapsdk.configuration import settings
from onapsdk.onap_service import OnapService


class CdsElement(OnapService, ABC):
    """Base CDS class.

    Stores url to CDS API (edit if you want to use other) and authentication tuple
    (username, password).
    """

    # These should be stored in configuration. There is even a task in Orange repo.
    _url: str = settings.CDS_URL
    auth: tuple = ("ccsdkapps", "ccsdkapps")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# SPDX-License-Identifier: Apache-2.0
"""Mixins module."""
from abc import ABC, abstractmethod
from ctypes import c_bool
from multiprocessing import Process, Value
from time import sleep


class WaitForFinishMixin(ABC):
    """Wait for finish mixin.

    Mixin with wait_for_finish method and two properties:
     - completed,
     - finished.

    Can be used to wait for result of asynchronous tasks.

    """

    WAIT_FOR_SLEEP_TIME = 10

    @property
    @abstractmethod
    def completed(self) -> bool:
        """Store an information if object task is completed or not.

        Returns:
            bool: True if object task is completed, False otherwise.

        """

    @property
    @abstractmethod
    def finished(self) -> bool:
        """Store an information if object task is finished or not.

        Returns:
            bool: True if object task is finished, False otherwise.

        """

    def _wait_for_finish(self, return_value: Value) -> bool:
        """Wait until object task is finished.

        Method called in another process.

        Args:
            return_value(Value): value shared with main process to pass there
                if object task was completed or not

        """
        while not self.finished:
            sleep(self.WAIT_FOR_SLEEP_TIME)
        self._logger.info(f"{self.__class__.__name__} task finished")
        return_value.value = self.completed

    def wait_for_finish(self, timeout: float = None) -> bool:
        """Wait until object task is finished.

        It uses time.sleep with WAIT_FOR_SLEEP_TIME value as a parameter to
            wait unitl request is finished (object's finished property is
            equal to True).

        It runs another process to control time of the function. If process timed out
            TimeoutError is going to be raised.

        Args:
            timeout(float, optional): positive number, wait at most timeout seconds

        Raises:
            TimeoutError: Raised when function timed out

        Returns:
            bool: True if object's task is successfully completed, False otherwise

        """
        self._logger.debug(f"Wait until {self.__class__.__name__} task is not finished")
        return_value: Value = Value(c_bool)
        wait_for_process: Process = Process(target=self._wait_for_finish, args=(return_value,))
        try:
            wait_for_process.start()
            wait_for_process.join(timeout)
            return return_value.value
        finally:
            if wait_for_process.is_alive():
                wait_for_process.terminate()
                raise TimeoutError



"""Connectivity-Info module."""
from onapsdk.utils.jinja import jinja_env
from ..msb_service import MSB


class ConnectivityInfo(MSB):
    """Connectivity-Info class."""

    api_version = "/api/multicloud-k8s/v1/v1"
    url = f"{MSB.base_url}{api_version}/connectivity-info"

    def __init__(self, cloud_region_id: str,
                 cloud_owner: str,
                 other_connectivity_list: dict,
                 kubeconfig: str) -> None:
        """Connectivity-info object initialization.

        Args:
            cloud_region_id (str): Cloud region ID
            cloud_owner (str): Cloud owner name
            other_connectivity_list (dict): Optional other connectivity list
            kubeconfig (str): kubernetes cluster kubeconfig
        """
        super().__init__()
        self.cloud_region_id: str = cloud_region_id
        self.cloud_owner: str = cloud_owner
        self.other_connectivity_list: dict = other_connectivity_list
        self.kubeconfig: str = kubeconfig

    @classmethod
    def get_connectivity_info_by_region_id(cls, cloud_region_id: str) -> "ConnectivityInfo":
        """Get connectivity-info by its name (cloud region id).

        Args:
            cloud_region_id (str): Cloud region ID

        Returns:
            ConnectivityInfo: Connectivity-Info object

        """
        url: str = f"{cls.url}/{cloud_region_id}"
        connectivity_info: dict = cls.send_message_json(
            "GET",
            "Get Connectivity Info",
            url
        )
        return cls(
            connectivity_info["cloud-region"],
            connectivity_info["cloud-owner"],
            connectivity_info.get("other-connectivity-list"),
            connectivity_info["kubeconfig"]
        )

    def delete(self) -> None:
        """Delete connectivity info."""
        url: str = f"{self.url}/{self.cloud_region_id}"
        self.send_message(
            "DELETE",
            "Delete Connectivity Info",
            url
        )

    @classmethod
    def create(cls,
               cloud_region_id: str,
               cloud_owner: str,
               kubeconfig: bytes = None) -> "ConnectivityInfo":
        """Create Connectivity Info.

        Args:
            cloud_region_id (str): Cloud region ID
            cloud_owner (str): Cloud owner name
            kubeconfig (bytes): kubernetes cluster kubeconfig file

        Returns:
            ConnectivityInfo: Created object

        """
        json_file = jinja_env().get_template("multicloud_k8s_add_connectivity_info.json.j2").render(
            cloud_region_id=cloud_region_id,
            cloud_owner=cloud_owner
        )
        url: str = f"{cls.url}"
        cls.send_message(
            "POST",
            "Create Connectivity Info",
            url,
            files={"file": kubeconfig,
                   "metadata": (None, json_file)},
            headers={}
        )
        return cls.get_connectivity_info_by_region_id(cloud_region_id)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# SPDX-License-Identifier: Apache-2.0
"""ONAP SDK utils package."""
import json
from datetime import datetime


def get_zulu_time_isoformat() -> str:
    """Get zulu time in accepted by ONAP modules format.

    Returns:
        str: Actual Zulu time.

    """
    return datetime.utcnow().strftime('%Y-%m-%dT%H:%M:%S.%fZ')


def load_json_file(path_to_json_file: str) -> str:
    """
    Return json as string from selected file.

    Args:
        path_to_json_file: (str) path to file with json
    Returns:
        File content as string (str)
    """
    with open(path_to_json_file) as json_file:
        data = json.load(json_file)
        return json.dumps(data)



"""A&AI owning entity module."""

from uuid import uuid4
from typing import Iterator

from onapsdk.utils.jinja import jinja_env
from onapsdk.exceptions import ResourceNotFound

from ..aai_element import AaiElement


class OwningEntity(AaiElement):
    """Owning entity class."""

    def __init__(self, name: str, owning_entity_id: str, resource_version: str) -> None:
        """Owning entity object initialization.

        Args:
            name (str): Owning entity name
            owning_entity_id (str): owning entity ID
            resource_version (str): resource version
        """
        super().__init__()
        self.name: str = name
        self.owning_entity_id: str = owning_entity_id
        self.resource_version: str = resource_version

    def __repr__(self) -> str:
        """Owning entity object representation.

        Returns:
            str: Owning entity object representation

        """
        return f"OwningEntity(name={self.name}, owning_entity_id={self.owning_entity_id})"

    @property
    def url(self) -> str:
        """Owning entity object url.

        Returns:
            str: Url

        """
        return (f"{self.base_url}{self.api_version}/business/owning-entities/owning-entity/"
                f"{self.owning_entity_id}?resource-version={self.resource_version}")

    @classmethod
    def get_all(cls) -> Iterator["OwningEntity"]:
        """Get all owning entities.

        Yields:
            OwningEntity: OwningEntity object

        """
        url: str = f"{cls.base_url}{cls.api_version}/business/owning-entities"
        for owning_entity in cls.send_message_json("GET",
                                                   "Get A&AI owning entities",
                                                   url).get("owning-entity", []):
            yield cls(
                owning_entity.get("owning-entity-name"),
                owning_entity.get("owning-entity-id"),
                owning_entity.get("resource-version")
            )

    @classmethod
    def get_by_owning_entity_id(cls, owning_entity_id: str) -> "OwningEntity":
        """Get owning entity by it's ID.

        Args:
            owning_entity_id (str): owning entity object id

        Returns:
            OwningEntity: OwningEntity object

        """
        response: dict = cls.send_message_json(
            "GET",
            "Get A&AI owning entity",
            (f"{cls.base_url}{cls.api_version}/business/owning-entities/"
             f"owning-entity/{owning_entity_id}")
        )
        return cls(
            response.get("owning-entity-name"),
            response.get("owning-entity-id"),
            response.get("resource-version")
        )

    @classmethod
    def get_by_owning_entity_name(cls, owning_entity_name: str) -> "OwningEntity":
        """Get owning entity resource by it's name.

        Raises:
            ResourceNotFound: Owning entity requested by a name does not exist.

        Returns:
            OwningEntity: Owning entity requested by a name.

        """
        for owning_entity in cls.get_all():
            if owning_entity.name == owning_entity_name:
                return owning_entity

        msg = f'Owning entity "{owning_entity_name}" does not exist.'
        raise ResourceNotFound(msg)

    @classmethod
    def create(cls, name: str, owning_entity_id: str = None) -> "OwningEntity":
        """Create owning entity A&AI resource.

        Args:
            name (str): owning entity name
            owning_entity_id (str): owning entity ID. Defaults to None.

        Returns:
            OwningEntity: Created OwningEntity object

        """
        if not owning_entity_id:
            owning_entity_id = str(uuid4())
        cls.send_message(
            "PUT",
            "Declare A&AI owning entity",
            (f"{cls.base_url}{cls.api_version}/business/owning-entities/"
             f"owning-entity/{owning_entity_id}"),
            data=jinja_env().get_template("aai_owning_entity_create.json.j2").render(
                owning_entity_name=name,
                owning_entity_id=owning_entity_id
            )
        )
        return cls.get_by_owning_entity_id(owning_entity_id)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# SPDX-License-Identifier: Apache-2.0
"""Database Adapter module."""
from abc import ABC
from dataclasses import dataclass

from onapsdk.so.so_element import SoElement
from onapsdk.onap_service import OnapService
from onapsdk.utils.headers_creator import headers_so_creator
from onapsdk.utils.jinja import jinja_env


@dataclass
class IdentityService:  # pylint: disable=too-many-instance-attributes
    """Class to store identity service details."""

    identity_id: str
    url: str = "http://1.2.3.4:5000/v2.0"
    mso_id: str = "onapsdk_user"
    mso_pass: str = "mso_pass_onapsdk"
    project_domain_name: str = "NULL"
    user_domain_name: str = "NULL"
    admin_tenant: str = "service"
    member_role: str = "admin"
    identity_server_type: str = "KEYSTONE"
    identity_authentication_type: str = "USERNAME_PASSWORD"
    hibernate_lazy_initializer = {}
    server_type_as_string: str = "KEYSTONE"
    tenant_metadata: bool = True


class SoDbAdapter(SoElement, ABC):
    """DB Adapter class."""

    @classmethod
    def add_cloud_site(cls,
                       cloud_region_id: str,
                       complex_id: str,
                       identity_service: IdentityService,
                       orchestrator: str = "multicloud"
                       ):
        """Add cloud_site data with identity_service to SO db.

        Args:
            cloud_region_id (str): The id of cloud region
            complex_id (str): The id of complex
            identity_service (IdentityService): Identity service related to the cloud region
            orchestrator (str, optional): Orchestrator type. Defaults to multicloud.

        Important:
            identity_services data will be overwrite, but in the same time
            cloud_sites data will not (shouldn't) be overwrite!
            SOCatalogDB REST API has some limitations reported: https://jira.onap.org/browse/SO-2727

        Return:
            response object
        """
        response = cls.send_message_json(
            "POST",
            "Create a region in SO db",
            f"{cls.base_url}/cloudSite",
            data=jinja_env().get_template("add_cloud_site_with_identity_service.json.j2").
            render(
                cloud_region_id=cloud_region_id,
                complex_id=complex_id,
                orchestrator=orchestrator,
                identity_service=identity_service
            ),
            headers=headers_so_creator(OnapService.headers)
        )
        return response



"""Pnf instance module."""

from typing import Optional, TYPE_CHECKING

from .instance import Instance

if TYPE_CHECKING:
    from .service import ServiceInstance  # pylint: disable=cyclic-import

class PnfInstance(Instance):  # pylint: disable=too-many-instance-attributes
    """Pnf instance class."""

    def __init__(self,  # pylint: disable=too-many-arguments, too-many-locals
                 service_instance: "ServiceInstance",
                 pnf_name: str,
                 in_maint: bool,
                 selflink: str = None,
                 pnf_id: str = None,
                 equip_type: str = None,
                 equip_vendor: str = None,
                 equip_model: str = None,
                 management_option: str = None,
                 orchestration_status: str = None,
                 ipaddress_v4_oam: str = None,
                 sw_version: str = None,
                 frame_id: str = None,
                 serial_number: str = None,
                 ipaddress_v4_loopback_0: str = None,
                 ipaddress_v6_loopback_0: str = None,
                 ipaddress_v4_aim: str = None,
                 ipaddress_v6_aim: str = None,
                 ipaddress_v6_oam: str = None,
                 inv_status: str = None,
                 resource_version: str = None,
                 prov_status: str = None,
                 nf_role: str = None,
                 admin_status: str = None,
                 operational_status: str = None,
                 model_customization_id: str = None,
                 model_invariant_id: str = None,
                 model_version_id: str = None,
                 pnf_ipv4_address: str = None,
                 pnf_ipv6_address: str = None) -> None:
        """Pnf instance object initialization.

        Args:
            service_instance (ServiceInstance):  Service instance object
            pnf_name (str): unique name of Physical Network Function
            in_maint (bool): Used to indicate whether or not this object is in maintenance mode
                (maintenance mode = True). This field (in conjunction with prov_status)
                is used to suppress alarms and vSCL on VNFs/VMs.
            selflink (str, optional): URL to endpoint where AAI can get more details.
                Defaults to None.
            pnf_id (str, optional): id of pnf. Defaults to None.
            equip_type (str, optional): Equipment type. Source of truth should define valid values.
                Defaults to None.
            equip_vendor (str, optional): Equipment vendor. Source of truth should define
                valid values. Defaults to None.
            equip_model (str, optional): Equipment model. Source of truth should define
                valid values. Defaults to None.
            management_option (str, optional): identifier of managed customer. Defaults to None.
            orchestration_status (str, optional): Orchestration status of this pnf.
                Defaults to None.
            ipaddress_v4_oam (str, optional): ipv4-oam-address with new naming
                convention for IP addresses. Defaults to None.
            sw_version (str, optional): sw-version is the version of SW for the hosted
                application on the PNF. Defaults to None.
            frame_id (str, optional): ID of the physical frame (relay rack) where pnf is installed.
                Defaults to None.
            serial_number (str, optional): Serial number of the device. Defaults to None.
            ipaddress_v4_loopback_0 (str, optional): IPV4 Loopback 0 address. Defaults to None.
            ipaddress_v6_loopback_0 (str, optional): IPV6 Loopback 0 address. Defaults to None.
            ipaddress_v4_aim (str, optional): IPV4 AIM address. Defaults to None.
            ipaddress_v6_aim (str, optional): IPV6 AIM address. Defaults to None.
            ipaddress_v6_oam (str, optional): IPV6 OAM address. Defaults to None.
            inv_status (str, optional): CANOPI's inventory status. Only set with values exactly
                as defined by CANOPI. Defaults to None.
            resource_version (str, optional): Used for optimistic concurrency.
                Must be empty on create, valid on update and delete. Defaults to None.
            prov_status (str, optional): Prov Status of this device (not under canopi control)
                Valid values [PREPROV/NVTPROV/PROV]. Defaults to None.
            nf_role (str, optional): Nf Role is the role performed by this instance in the network.
                Defaults to None.
            admin_status (str, optional): admin Status of this PNF. Defaults to None.
            operational_status (str, optional): Store the operational-status for this object.
                Defaults to None.
            model_customization_id (str, optional): Store the model-customization-id
                for this object. Defaults to None.
            model_invariant_id (str, optional): The ASDC model id for this resource model.
                Defaults to None.
            model_version_id (str, optional): The ASDC model version for this resource model.
                Defaults to None.
            pnf_ipv4_address (str, optional): This is the IP address (IPv4) for the PNF itself.
                This is the IPv4 address that the PNF iself can be accessed at. Defaults to None.
            pnf_ipv6_address (str, optional): This is the IP address (IPv6) for the PNF itself.
                This is the IPv6 address that the PNF iself can be accessed at. Defaults to None.
        """
        super().__init__(resource_version=resource_version,
                         model_invariant_id=model_invariant_id,
                         model_version_id=model_version_id)
        self.service_instance: "ServiceInstance" = service_instance
        self.pnf_name: str = pnf_name
        self.in_maint: bool = in_maint
        self.selflink: Optional[str] = selflink
        self.pnf_id: Optional[str] = pnf_id
        self.equip_type: Optional[str] = equip_type
        self.equip_vendor: Optional[str] = equip_vendor
        self.equip_model: Optional[str] = equip_model
        self.management_option: Optional[str] = management_option
        self.orchestration_status: Optional[str] = orchestration_status
        self.ipaddress_v4_oam: Optional[str] = ipaddress_v4_oam
        self.sw_version: Optional[str] = sw_version
        self.frame_id: Optional[str] = frame_id
        self.serial_number: Optional[str] = serial_number
        self.ipaddress_v4_loopback_0: Optional[str] = ipaddress_v4_loopback_0
        self.ipaddress_v6_loopback_0: Optional[str] = ipaddress_v6_loopback_0
        self.ipaddress_v4_aim: Optional[str] = ipaddress_v4_aim
        self.ipaddress_v6_aim: Optional[str] = ipaddress_v6_aim
        self.ipaddress_v6_oam: Optional[str] = ipaddress_v6_oam
        self.inv_status: Optional[str] = inv_status
        self.prov_status: Optional[str] = prov_status
        self.nf_role: Optional[str] = nf_role
        self.admin_status: Optional[str] = admin_status
        self.operational_status: Optional[str] = operational_status
        self.model_customization_id: Optional[str] = model_customization_id
        self.pnf_ipv4_address: Optional[str] = pnf_ipv4_address
        self.pnf_ipv6_address: Optional[str] = pnf_ipv6_address

    def __repr__(self) -> str:
        """Pnf instance object representation.

        Returns:
            str: Human readable pnf instance representation

        """
        return f"PnfInstance(pnf_name={self.pnf_name})"

    @property
    def url(self) -> str:
        """Network instance url.

        Returns:
            str: NetworkInstance url

        """
        return f"{self.base_url}{self.api_version}/network/pnfs/pnf/{self.pnf_name}"

    @classmethod
    def create_from_api_response(cls, api_response: dict,
                                 service_instance: "ServiceInstance") -> "PnfInstance":
        """Create pnf instance object using HTTP API response dictionary.

        Args:
            api_response (dict): A&AI API response dictionary
            service_instance (ServiceInstance): Service instance with which network is related

        Returns:
            PnfInstance: PnfInstance object

        """
        return cls(service_instance=service_instance,
                   pnf_name=api_response["pnf-name"],
                   in_maint=api_response["in-maint"],
                   selflink=api_response.get("selflink"),
                   pnf_id=api_response.get("pnf-id"),
                   equip_type=api_response.get("equip-type"),
                   equip_vendor=api_response.get("equip-vendor"),
                   equip_model=api_response.get("equip-model"),
                   management_option=api_response.get("management-option"),
                   orchestration_status=api_response.get("orchestration-status"),
                   ipaddress_v4_oam=api_response.get("ipaddress-v4-oam"),
                   sw_version=api_response.get("sw-version"),
                   frame_id=api_response.get("frame-id"),
                   serial_number=api_response.get("serial-number"),
                   ipaddress_v4_loopback_0=api_response.get("ipaddress-v4-loopback-0"),
                   ipaddress_v6_loopback_0=api_response.get("ipaddress-v6-loopback-0"),
                   ipaddress_v4_aim=api_response.get("ipaddress-v4-aim"),
                   ipaddress_v6_aim=api_response.get("ipaddress-v6-aim"),
                   ipaddress_v6_oam=api_response.get("ipaddress-v6-oam"),
                   inv_status=api_response.get("inv-status"),
                   resource_version=api_response.get("resource-version"),
                   prov_status=api_response.get("prov-status"),
                   nf_role=api_response.get("nf-role"),
                   admin_status=api_response.get("admin-status"),
                   operational_status=api_response.get("operational-status"),
                   model_customization_id=api_response.get("model-customization-id"),
                   model_invariant_id=api_response.get("model-invariant-id"),
                   model_version_id=api_response.get("model-version-id"),
                   pnf_ipv4_address=api_response.get("pnf-ipv4-address"),
                   pnf_ipv6_address=api_response.get("pnf-ipv6-address"))

    def delete(self) -> None:
        """Delete Pnf instance.

        PNF deletion it's just A&AI resource deletion. That's difference between another instances.
        You don't have to wait for that task finish, because it's not async task.

        """
        self._logger.debug("Delete %s pnf", self.pnf_name)
        self.send_message("DELETE",
                          f"Delete {self.pnf_name} PNF",
                          f"{self.url}?resource-version={self.resource_version}")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# SPDX-License-Identifier: Apache-2.0
"""SO Element module."""
import json
from abc import ABC
from dataclasses import dataclass
from enum import Enum
from typing import Dict

from onapsdk.configuration import settings
from onapsdk.sdc.service import Service
from onapsdk.sdc.vf import Vf
from onapsdk.onap_service import OnapService
from onapsdk.utils.headers_creator import headers_so_creator
from onapsdk.utils.jinja import jinja_env
from onapsdk.utils.mixins import WaitForFinishMixin
from onapsdk.utils.tosca_file_handler import get_modules_list_from_tosca_file


@dataclass
class SoElement(OnapService):
    """Mother Class of all SO elements."""

    name: str = None
    _server: str = "SO"
    base_url = settings.SO_URL
    api_version = settings.SO_API_VERSION
    _status: str = None

    @property
    def headers(self):
        """Create headers for SO request.

        It is used as a property because x-transactionid header should be unique for each request.
        """
        return headers_so_creator(OnapService.headers)

    @classmethod
    def get_subscription_service_type(cls, vf_name):
        """Retrieve the model info of the VFs."""
        vf_object = Vf(name=vf_name)
        return vf_object.name

    @classmethod
    def get_service_model_info(cls, service_name):
        """Retrieve Service Model info."""
        service = Service(name=service_name)
        template_service = jinja_env().get_template("service_instance_model_info.json.j2")
        # Get service instance model
        parsed = json.loads(
            template_service.render(
                model_invariant_id=service.unique_uuid,
                model_name_version_id=service.identifier,
                model_name=service.name,
                model_version=service.version,
            )
        )
        return json.dumps(parsed, indent=4)

    @classmethod
    def get_vnf_model_info(cls, vf_name):
        """Retrieve the model info of the VFs."""
        vf_object = Vf(name=vf_name)
        template_service = jinja_env().get_template("vnf_model_info.json.j2")
        parsed = json.loads(
            template_service.render(
                vnf_model_invariant_uuid=vf_object.unique_uuid,
                vnf_model_customization_id="????",
                vnf_model_version_id=vf_object.identifier,
                vnf_model_name=vf_object.name,
                vnf_model_version=vf_object.version,
                vnf_model_instance_name=(vf_object.name + " 0"),
            )
        )
        # we need also a vnf instance Name
        # Usually it is found like that
        # name: toto
        # instance name: toto 0
        # it can be retrieved from the tosca
        return json.dumps(parsed, indent=4)

    @classmethod
    def get_vf_model_info(cls, vf_model: str) -> str:
        """Retrieve the VF model info From Tosca?."""
        modules: Dict = get_modules_list_from_tosca_file(vf_model)
        template_service = jinja_env().get_template("vf_model_info.json.j2")
        parsed = json.loads(template_service.render(modules=modules))
        return json.dumps(parsed, indent=4)

    @classmethod
    def _base_create_url(cls) -> str:
        """
        Give back the base url of SO.

        Returns:
            str: the base url

        """
        return "{}/onap/so/infra/serviceInstantiation/{}/serviceInstances".format(
            cls.base_url, cls.api_version
        )


class OrchestrationRequest(SoElement, WaitForFinishMixin, ABC):
    """Base SO orchestration request class."""

    WAIT_FOR_SLEEP_TIME = 10

    def __init__(self,
                 request_id: str) -> None:
        """Instantiate object initialization.

        Initializator used by classes inherited from this abstract class.

        Args:
            request_id (str): request ID
        """
        super().__init__()
        self.request_id: str = request_id

    class StatusEnum(Enum):
        """Status enum.

        Store possible statuses for instantiation:
            - IN_PROGRESS,
            - FAILED,
            - COMPLETE.
        If instantiation has status which is not covered by these values
            UNKNOWN value is used.

        """

        IN_PROGRESS = "IN_PROGRESS"
        FAILED = "FAILED"
        COMPLETED = "COMPLETE"
        UNKNOWN = "UNKNOWN"

    @property
    def status(self) -> "StatusEnum":
        """Object instantiation status.

        It's populated by call SO orchestation request endpoint.

        Returns:
            StatusEnum: Instantiation status.

        """
        response: dict = self.send_message_json(
            "GET",
            f"Check {self.request_id} orchestration request status",
            (f"{self.base_url}/onap/so/infra/"
             f"orchestrationRequests/{self.api_version}/{self.request_id}"),
            headers=headers_so_creator(OnapService.headers)
        )
        try:
            return self.StatusEnum(response["request"]["requestStatus"]["requestState"])
        except (KeyError, ValueError):
            self._logger.exception("Invalid status")
            return self.StatusEnum.UNKNOWN

    @property
    def finished(self) -> bool:
        """Store an information if instantion is finished or not.

        Instantiation is finished if it's status is COMPLETED or FAILED.

        Returns:
            bool: True if instantiation is finished, False otherwise.

        """
        return self.status in [self.StatusEnum.COMPLETED, self.StatusEnum.FAILED]

    @property
    def completed(self) -> bool:
        """Store an information if instantion is completed or not.

        Instantiation is completed if it's status is COMPLETED.

        Returns:
            bool: True if instantiation is completed, False otherwise.

        """
        return self.finished and self.status == self.StatusEnum.COMPLETED

    @property
    def failed(self) -> bool:
        """Store an information if instantion is failed or not.

        Instantiation is failed if it's status is FAILED.

        Returns:
            bool: True if instantiation is failed, False otherwise.

        """
        return self.finished and self.status == self.StatusEnum.FAILED



# SPDX-License-Identifier: Apache-2.0
# Copyright 2020 Nokia
"""Base VES event sender."""
from typing import Dict, Union

import json
import requests

from onapsdk.ves.ves_service import VesService

ACTION = "Send event to Ves"
POST_HTTP_METHOD = "POST"


class Ves(VesService):
    """Ves library provides functions for sending events to VES."""

    event_endpoint_url: str = "{}/eventListener/{}"
    event_batch_endpoint_url: str = "{}/eventListener/{}/eventBatch"

    @classmethod
    def send_event(cls,
                   version: str,
                   json_event: str,
                   basic_auth: Dict[str, str]) -> Union[requests.Response, None]:
        """
        Send an event stored in a file to VES.

        Args:
            version: (str) version of VES data format
            json_event: (str) event to send
            basic_auth: Dict[str, str], for example:{ 'username': 'bob', 'password': 'secret' }
        Returns:
            (requests.Response) HTTP response status

        """
        return Ves.__send_event_message(cls.event_endpoint_url.format(VesService._url, version),
                                        json_event, basic_auth)

    @classmethod
    def send_batch_event(cls,
                         version: str,
                         json_event: str,
                         basic_auth: Dict[str, str]) -> Union[requests.Response, None]:
        """
        Send a batch event stored in a file to VES.

        Args:
            version: (str) version of VES data format
            json_event: (str) event to send
            basic_auth: Dict[str, str], for example:{ 'username': 'bob', 'password': 'secret' }
        Returns:
            (requests.Response) HTTP response status

        """
        return Ves.__send_event_message(cls.event_batch_endpoint_url.
                                        format(VesService._url, version),
                                        json_event, basic_auth)

    @classmethod
    def __send_event_message(cls,
                             base_url: str,
                             json_event: str,
                             basic_auth: Dict[str, str]
                             ) -> Union[requests.Response, None]:
        cls._logger.debug("Event to send %s", json_event)
        return cls.send_message(
            POST_HTTP_METHOD,
            ACTION,
            f"{base_url}",
            basic_auth=basic_auth,
            json=json.loads(json_event)
        )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# SPDX-License-Identifier: Apache-2.0
"""Pnf module."""
from typing import Dict, List, Union
from onapsdk.exceptions import ParameterError

from onapsdk.sdc.sdc_resource import SdcResource
from onapsdk.sdc.properties import NestedInput, Property
import onapsdk.constants as const
from onapsdk.sdc.vendor import Vendor
from onapsdk.sdc.vsp import Vsp


class Pnf(SdcResource):
    """
    ONAP PNF Object used for SDC operations.

    Attributes:
        name (str): the name of the pnf. Defaults to "ONAP-test-PNF".
        identifier (str): the unique ID of the pnf from SDC.
        status (str): the status of the pnf from SDC.
        version (str): the version ID of the vendor from SDC.
        uuid (str): the UUID of the PNF (which is different from identifier,
                    don't ask why...)
        unique_identifier (str): Yet Another ID, just to puzzle us...
        vendor (optional): the vendor of the PNF
        vsp (optional): the vsp related to the PNF

    """

    def __init__(self, name: str = None, version: str = None, vendor: Vendor = None, # pylint: disable=too-many-arguments
                 sdc_values: Dict[str, str] = None, vsp: Vsp = None,
                 properties: List[Property] = None, inputs: Union[Property, NestedInput] = None,
                 category: str = None, subcategory: str = None):
        """
        Initialize pnf object.

        Args:
            name (optional): the name of the pnf
            version (str, optional): the version of a PNF object

        """
        super().__init__(sdc_values=sdc_values, version=version, properties=properties,
                         inputs=inputs, category=category, subcategory=subcategory)
        self.name: str = name or "ONAP-test-PNF"
        self.vendor: Vendor = vendor
        self.vsp: Vsp = vsp

    def create(self) -> None:
        """Create the PNF in SDC if not already existing."""
        if not self.vsp and not self.vendor:
            raise ParameterError("Neither Vsp nor Vendor provided.")
        self._create("pnf_create.json.j2",
                     name=self.name,
                     vsp=self.vsp,
                     vendor=self.vendor,
                     category=self.category)

    def _really_submit(self) -> None:
        """Really submit the SDC PNF in order to enable it."""
        result = self._action_to_sdc(const.CERTIFY, "lifecycleState")
        if result:
            self.load()



"""AAI service-design-and-creation module."""
from typing import Iterator
from urllib.parse import urlencode

from onapsdk.utils.jinja import jinja_env

from .aai_element import AaiElement


class Service(AaiElement):
    """SDC service class."""

    def __init__(self, service_id: str, service_description: str, resource_version: str) -> None:
        """Service model initialization.

        Args:
            service_id (str): This gets defined by others to provide a unique ID for the service.
            service_description (str): Description of the service.
            resource_version (str): Used for optimistic concurrency.

        """
        super().__init__()
        self.service_id = service_id
        self.service_description = service_description
        self.resource_version = resource_version

    def __repr__(self) -> str:
        """Service object description.

        Returns:
            str: Service object description

        """
        return (
            f"Service(service_id={self.service_id}, "
            f"service_description={self.service_description}, "
            f"resource_version={self.resource_version})"
        )

    @property
    def url(self) -> str:
        """Service object url.

        Returns:
            str: Service object url address

        """
        return (f"{self.base_url}{self.api_version}/service-design-and-creation/services/service/"
                f"{self.service_id}?resource-version={self.resource_version}")

    @classmethod
    def get_all(cls,
                service_id: str = None,
                service_description: str = None) -> Iterator["Service"]:
        """Services iterator.

        Stand-in for service model definitions.

        Returns:
            Iterator[Service]: Service

        """
        filter_parameters: dict = cls.filter_none_key_values(
            {"service-id": service_id, "service-description": service_description}
        )
        url: str = (f"{cls.base_url}{cls.api_version}/service-design-and-creation/"
                    f"services?{urlencode(filter_parameters)}")
        for service in cls.send_message_json("GET", "get subscriptions", url).get("service", []):
            yield Service(
                service_id=service["service-id"],
                service_description=service["service-description"],
                resource_version=service["resource-version"],
            )

    @classmethod
    def create(cls,
               service_id: str,
               service_description: str) -> None:
        """Create service.

        Args:
            service_id (str): service ID
            service_description (str): service description

        """
        cls.send_message(
            "PUT",
            "Create A&AI service",
            f"{cls.base_url}{cls.api_version}/service-design-and-creation/"
            f"services/service/{service_id}",
            data=jinja_env()
            .get_template("aai_service_create.json.j2")
            .render(
                service_id=service_id,
                service_description=service_description
            )
        )


class Model(AaiElement):
    """Model resource class."""

    def __init__(self, invariant_id: str, model_type: str, resource_version: str) -> None:
        """Model object initialization.

        Args:
            invariant_id (str): invariant id
            model_type (str): model type
            resource_version (str): resource version

        """
        super().__init__()
        self.invariant_id: str = invariant_id
        self.model_type: str = model_type
        self.resource_version: str = resource_version

    def __repr__(self) -> str:
        """Model object representation.

        Returns:
            str: model object representation

        """
        return (f"Model(invatiant_id={self.invariant_id}, "
                f"model_type={self.model_type}, "
                f"resource_version={self.resource_version}")

    @property
    def url(self) -> str:
        """Model instance url.

        Returns:
            str: Model's url

        """
        return (f"{self.base_url}{self.api_version}/service-design-and-creation/models/"
                f"model/{self.invariant_id}?resource-version={self.resource_version}")

    @classmethod
    def get_all(cls) -> Iterator["Model"]:
        """Get all models.

        Yields:
            Model: Model object

        """
        for model in cls.send_message_json("GET",
                                           "Get A&AI sdc models",
                                           (f"{cls.base_url}{cls.api_version}/"
                                            "service-design-and-creation/models")).get("model",
                                                                                       []):
            yield Model(
                invariant_id=model.get("model-invariant-id"),
                model_type=model.get("model-type"),
                resource_version=model.get("resource-version")
            )



"""A&AI project module."""

from typing import Any, Dict, Iterator

from onapsdk.utils.jinja import jinja_env

from ..aai_element import AaiElement


class Project(AaiElement):
    """Project class."""

    def __init__(self, name: str, resource_version: str) -> None:
        """Project object initialization.

        Args:
            name (str): Project name
            resource_version (str): resource version
        """
        super().__init__()
        self.name: str = name
        self.resource_version: str = resource_version

    @classmethod
    def get_all(cls) -> Iterator["Project"]:
        """Get all project.

        Yields:
            Project: Project object

        """
        url: str = f"{cls.base_url}{cls.api_version}/business/projects"
        for project in cls.send_message_json("GET",
                                             "Get A&AI projects",
                                             url).get("project", []):
            yield cls(
                project.get("project-name"),
                project.get("resource-version")
            )

    def __repr__(self) -> str:
        """Project object representation.

        Returns:
            str: Project object representation

        """
        return f"Project(name={self.name})"

    @property
    def url(self) -> str:
        """Project's url.

        Returns:
            str: Resource's url

        """
        return (f"{self.base_url}{self.api_version}/business/projects/"
                f"project/{self.name}")

    @classmethod
    def create(cls, name: str) -> "Project":
        """Create project A&AI resource.

        Args:
            name (str): project name

        Returns:
            Project: Created Project object

        """
        cls.send_message(
            "PUT",
            "Declare A&AI project",
            (f"{cls.base_url}{cls.api_version}/business/projects/"
             f"project/{name}"),
            data=jinja_env().get_template("aai_project_create.json.j2").render(
                project_name=name
            )
        )
        return cls.get_by_name(name)

    @classmethod
    def get_by_name(cls, name: str) -> "Project":
        """Get project resource by it's name.

        Raises:
            ResourceNotFound: Project requested by a name does not exist.

        Returns:
            Project: Project requested by a name.

        """
        url = (f"{cls.base_url}{cls.api_version}/business/projects/"
               f"project/{name}")
        response: Dict[str, Any] = \
            cls.send_message_json("GET",
                                  f"Get {name} project",
                                  url)
        return cls(response["project-name"], response["resource-version"])



"""A&AI Tenant module."""
from ..aai_element import AaiElement


class Tenant(AaiElement):
    """Tenant class."""

    def __init__(self,  # pylint: disable=too-many-arguments
                 cloud_region: "CloudRegion",
                 tenant_id: str,
                 tenant_name: str,
                 tenant_context: str = None,
                 resource_version: str = None):
        """Tenant object initialization.

        Tenant object represents A&AI Tenant resource.

        Args:
            cloud_region (str): Cloud region object
            tenant_id (str): Unique Tenant ID
            tenant_name (str): Tenant name
            tenant_context (str, optional): Tenant context. Defaults to None.
            resource_version (str, optional): Tenant resource version. Defaults to None.

        """
        super().__init__()
        self.cloud_region: "CloudRegion" = cloud_region
        self.tenant_id: str = tenant_id
        self.name: str = tenant_name
        self.context: str = tenant_context
        self.resource_version: str = resource_version

    def __repr__(self) -> str:
        """Tenant repr.

        Returns:
            str: Human readable Tenant object description

        """
        return (
            f"Tenant(tenant_id={self.tenant_id}, tenant_name={self.name}, "
            f"tenant_context={self.context}, "
            f"resource_version={self.resource_version}, "
            f"cloud_region={self.cloud_region.cloud_region_id})"
        )

    @property
    def url(self) -> str:
        """Tenant url.

        Returns:
            str: Url which can be used to update or delete tenant.

        """
        return (
            f"{self.base_url}{self.api_version}/cloud-infrastructure/cloud-regions/cloud-region/"
            f"{self.cloud_region.cloud_owner}/{self.cloud_region.cloud_region_id}"
            f"/tenants/tenant/{self.tenant_id}?"
            f"resource-version={self.resource_version}"
        )

    def delete(self) -> None:
        """Delete tenant.

        Remove tenant from cloud region.

        """
        return self.send_message(
            "DELETE",
            f"Remove tenant {self.name} from {self.cloud_region.cloud_region_id} cloud region",
            url=self.url,
        )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# SPDX-License-Identifier: Apache-2.0
"""NBI module."""
from abc import ABC
from enum import Enum
from typing import Iterator
from uuid import uuid4

from onapsdk.aai.business.customer import Customer
from onapsdk.exceptions import RequestError
from onapsdk.onap_service import OnapService
from onapsdk.utils import get_zulu_time_isoformat
from onapsdk.utils.jinja import jinja_env
from onapsdk.utils.mixins import WaitForFinishMixin
from onapsdk.configuration import settings


class Nbi(OnapService, ABC):
    """NBI base class."""

    base_url = settings.NBI_URL
    api_version = settings.NBI_API_VERSION

    @classmethod
    def is_status_ok(cls) -> bool:
        """Check NBI service status.

        Returns:
            bool: True if NBI works fine, False otherwise

        """
        try:
            cls.send_message(
                "GET",
                "Check NBI status",
                f"{cls.base_url}{cls.api_version}/status"
            )
        except RequestError as exc:
            msg = f"An error occured during NBI status check: {exc}"
            cls._logger.error(msg)
            return False
        return True


class ServiceSpecification(Nbi):
    """NBI service specification class."""

    def __init__(self,  # pylint: disable=too-many-arguments
                 unique_id: str,
                 name: str,
                 invariant_uuid: str,
                 category: str,
                 distribution_status: str,
                 version: str,
                 lifecycle_status: str) -> None:
        """Service specification object initialization.

        Args:
            unique_id (str): Unique ID
            name (str): Service specification name
            invariant_uuid (str): Invariant UUID
            category (str): Category
            distribution_status (str): Service distribution status
            version (str): Service version
            lifecycle_status (str): Service lifecycle status
        """
        super().__init__()
        self.unique_id: str = unique_id
        self.name: str = name
        self.invariant_uuid: str = invariant_uuid
        self.category: str = category
        self.distribution_status: str = distribution_status
        self.version: str = version
        self.lifecycle_status: str = lifecycle_status

    def __repr__(self) -> str:
        """Service specification representation.

        Returns:
            str: Service specification object human readable representation

        """
        return (f"ServiceSpecification(unique_id={self.unique_id}, name={self.name}, "
                f"invariant_uuid={self.invariant_uuid}, category={self.category}, "
                f"distribution_status={self.distribution_status}, version={self.version}, "
                f"lifecycle_status={self.lifecycle_status})")

    @classmethod
    def get_all(cls) -> Iterator["ServiceSpecification"]:
        """Get all service specifications.

        Yields:
            ServiceSpecification: Service specification object

        """
        for service_specification in cls.send_message_json("GET",
                                                           "Get service specifications from NBI",
                                                           (f"{cls.base_url}{cls.api_version}/"
                                                            "serviceSpecification")):
            yield ServiceSpecification(
                service_specification.get("id"),
                service_specification.get("name"),
                service_specification.get("invariantUUID"),
                service_specification.get("category"),
                service_specification.get("distributionStatus"),
                service_specification.get("version"),
                service_specification.get("lifecycleStatus"),
            )

    @classmethod
    def get_by_id(cls, service_specification_id: str) -> "ServiceSpecification":
        """Get service specification by ID.

        Args:
            service_specification_id (str): Service specification ID

        Returns:
            ServiceSpecification: Service specification object

        """
        service_specification: dict = cls.send_message_json(
            "GET",
            f"Get service specification with {service_specification_id} ID from NBI",
            f"{cls.base_url}{cls.api_version}/serviceSpecification/{service_specification_id}"
        )
        return ServiceSpecification(
            service_specification.get("id"),
            service_specification.get("name"),
            service_specification.get("invariantUUID"),
            service_specification.get("category"),
            service_specification.get("distributionStatus"),
            service_specification.get("version"),
            service_specification.get("lifecycleStatus"),
        )


class Service(Nbi):
    """NBI service."""

    def __init__(self,  # pylint: disable=too-many-arguments
                 name: str,
                 service_id: str,
                 service_specification_name: str,
                 service_specification_id: str,
                 customer_id: str,
                 customer_role: str,
                 href: str) -> None:
        """Service object initialization.

        Args:
            name (str): Service name
            service_id (str): Service ID
            service_specification_name (str): Service specification name
            service_specification_id (str): Service specification ID
            customer_id (str): Global customer ID
            customer_role (str): Customer role
            href (str): Service object href
        """
        super().__init__()
        self.name: str = name
        self.service_id: str = service_id
        self._service_specification_name: str = service_specification_name
        self._service_specification_id: str = service_specification_id
        self._customer_id: str = customer_id
        self.customer_role: str = customer_role
        self.href: str = href

    def __repr__(self) -> str:
        """Service object representation.

        Returns:
            str: Human readable service object representation

        """
        return (f"Service(name={self.name}, service_id={self.service_id}, "
                f"service_specification={self.service_specification}, customer={self.customer}, "
                f"customer_role={self.customer_role})")

    @classmethod
    def get_all(cls, customer_id: str = 'generic') -> Iterator["Service"]:
        """Get all services for selected customer.

        Args:
            customer_id (str): Global customer ID

        Yields:
            Service: Service object

        """
        for service in cls.send_message_json("GET",
                                             "Get service instances from NBI",
                                             f"{cls.base_url}{cls.api_version}/service?"
                                             f"relatedParty.id={customer_id}"):
            yield cls(service.get("name"),
                      service.get("id"),
                      service.get("serviceSpecification", {}).get("name"),
                      service.get("serviceSpecification", {}).get("id"),
                      service.get("relatedParty", {}).get("id"),
                      service.get("relatedParty", {}).get("role"),
                      service.get("href"))

    @property
    def customer(self) -> Customer:
        """Service order Customer object.

        Returns:
            Customer: Customer object

        """
        if not self._customer_id:
            return None
        return Customer.get_by_global_customer_id(self._customer_id)

    @property
    def service_specification(self) -> ServiceSpecification:
        """Service specification.

        Returns:
            ServiceSpecification: Service specification object

        """
        if not self._service_specification_id:
            return None
        return ServiceSpecification.get_by_id(self._service_specification_id)


class ServiceOrder(Nbi, WaitForFinishMixin):  # pylint: disable=too-many-instance-attributes
    """Service order class."""

    WAIT_FOR_SLEEP_TIME = 10

    def __init__(self,  # pylint: disable=too-many-arguments
                 unique_id: str,
                 href: str,
                 priority: str,
                 description: str,
                 category: str,
                 external_id: str,
                 service_instance_name: str,
                 state: str = None,
                 customer: Customer = None,
                 customer_id: str = None,
                 service_specification: ServiceSpecification = None,
                 service_specification_id: str = None) -> None:
        """Service order object initialization.

        Args:
            unique_id (str): unique ID
            href (str): object's href
            priority (str): order priority
            description (str): order description
            category (str): category description
            external_id (str): external ID
            service_instance_name (str): name of service instance
            state (str, optional): instantiation state. Defaults to None.
            customer (Customer, optional): Customer object. Defaults to None.
            customer_id (str, optional): global customer ID. Defaults to None.
            service_specification (ServiceSpecification, optional): service specification object.
                Defaults to None.
            service_specification_id (str, optional): service specification ID. Defaults to None.
        """
        super().__init__()
        self.unique_id: str = unique_id
        self.href: str = href
        self.priority: str = priority
        self.category: str = category
        self.description: str = description
        self.external_id: str = external_id
        self._customer: Customer = customer
        self._customer_id: str = customer_id
        self._service_specification: ServiceSpecification = service_specification
        self._service_specification_id: str = service_specification_id
        self.service_instance_name: str = service_instance_name
        self.state: str = state

    class StatusEnum(Enum):
        """Status enum.

        Store possible statuses for service order:
            - completed,
            - failed,
            - inProgress.
        If instantiation has status which is not covered by these values
            `unknown` value is used.

        """

        ACKNOWLEDGED = "acknowledged"
        IN_PROGRESS = "inProgress"
        FAILED = "failed"
        COMPLETED = "completed"
        REJECTED = "rejected"
        UNKNOWN = "unknown"

    def __repr__(self) -> str:
        """Service order object representation.

        Returns:
            str: Service order object representation.

        """
        return (f"ServiceOrder(unique_id={self.unique_id}, href={self.href}, "
                f"priority={self.priority}, category={self.category}, "
                f"description={self.description}, external_id={self.external_id}, "
                f"customer={self.customer}, service_specification={self.service_specification}"
                f"service_instance_name={self.service_instance_name}, state={self.state})")

    @property
    def customer(self) -> Customer:
        """Get customer object used in service order.

        Returns:
            Customer: Customer object

        """
        if not self._customer:
            if not self._customer_id:
                self._logger.error("No customer ID")
                return None
            self._customer = Customer.get_by_global_customer_id(self._customer_id)
        return self._customer

    @property
    def service_specification(self) -> ServiceSpecification:
        """Service order service specification used in order item.

        Returns:
            ServiceSpecification: Service specification

        """
        if not self._service_specification:
            if not self._service_specification_id:
                self._logger.error("No service specification")
                return None
            self._service_specification = ServiceSpecification.\
                get_by_id(self._service_specification_id)
        return self._service_specification

    @classmethod
    def get_all(cls) -> Iterator["ServiceOrder"]:
        """Get all service orders.

        Returns:
            Iterator[ServiceOrder]: ServiceOrder object

        """
        for service_order in cls.send_message_json("GET",
                                                   "Get all service orders",
                                                   f"{cls.base_url}{cls.api_version}/serviceOrder"):
            service_order_related_party = None
            if service_order.get("relatedParty") is not None:
                service_order_related_party = service_order.get(
                    "relatedParty", [{}])[0].get("id")

            yield ServiceOrder(
                unique_id=service_order.get("id"),
                href=service_order.get("href"),
                priority=service_order.get("priority"),
                category=service_order.get("category"),
                description=service_order.get("description"),
                external_id=service_order.get("externalId"),
                customer_id=service_order_related_party,
                service_specification_id=service_order.get("orderItem", [{}])[0].get("service")\
                    .get("serviceSpecification").get("id"),
                service_instance_name=service_order.get("orderItem", [{}])[0].\
                    get("service", {}).get("name"),
                state=service_order.get("state")
            )

    @classmethod
    def create(cls,
               customer: Customer,
               service_specification: ServiceSpecification,
               name: str = None,
               external_id: str = None) -> "ServiceOrder":
        """Create service order.

        Returns:
            ServiceOrder: ServiceOrder object

        """
        if external_id is None:
            external_id = str(uuid4())
        if name is None:
            name = f"Python_ONAP_SDK_service_instance_{str(uuid4())}"
        response: dict = cls.send_message_json(
            "POST",
            "Add service instance via ServiceOrder API",
            f"{cls.base_url}{cls.api_version}/serviceOrder",
            data=jinja_env()
            .get_template("nbi_service_order_create.json.j2")
            .render(
                customer=customer,
                service_specification=service_specification,
                service_instance_name=name,
                external_id=external_id,
                request_time=get_zulu_time_isoformat()
            )
        )
        return cls(
            unique_id=response.get("id"),
            href=response.get("href"),
            priority=response.get("priority"),
            description=response.get("description"),
            category=response.get("category"),
            external_id=response.get("externalId"),
            customer=customer,
            service_specification=service_specification,
            service_instance_name=name
        )

    @property
    def status(self) -> "StatusEnum":
        """Service order instantiation status.

        It's populated by call Service order endpoint.

        Returns:
            StatusEnum: Service order status.

        """
        response: dict = self.send_message_json("GET",
                                                "Get service order status",
                                                (f"{self.base_url}{self.api_version}/"
                                                 f"serviceOrder/{self.unique_id}"))
        try:
            return self.StatusEnum(response.get("state"))
        except (KeyError, ValueError):
            self._logger.exception("Invalid status")
            return self.StatusEnum.UNKNOWN

    @property
    def completed(self) -> bool:
        """Store an information if service order is completed or not.

        Service orded is completed if it's status is COMPLETED.

        Returns:
            bool: True if service orded is completed, False otherwise.

        """
        return self.status == self.StatusEnum.COMPLETED

    @property
    def rejected(self) -> bool:
        """Store an information if service order is rejected or not.

        Service orded is completed if it's status is REJECTED.

        Returns:
            bool: True if service orded is rejected, False otherwise.

        """
        return self.status == self.StatusEnum.REJECTED

    @property
    def failed(self) -> bool:
        """Store an information if service order is failed or not.

        Service orded is completed if it's status is FAILED.

        Returns:
            bool: True if service orded is failed, False otherwise.

        """
        return self.status == self.StatusEnum.FAILED

    @property
    def finished(self) -> bool:
        """Store an information if service order is finished or not.

        Service orded is finished if it's status is not ACKNOWLEDGED or IN_PROGRESS.

        Returns:
            bool: True if service orded is finished, False otherwise.

        """
        return self.status not in [self.StatusEnum.ACKNOWLEDGED,
                                   self.StatusEnum.IN_PROGRESS]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# SPDX-License-Identifier: Apache-2.0
"""ONAP Service module."""
from abc import ABC
from dataclasses import dataclass, field
from typing import Any, Callable, Dict, Iterator, List, Optional, Union

import logging
import requests
import urllib3
from urllib3.util.retry import Retry
import simplejson.errors

from requests.adapters import HTTPAdapter
from requests import (  # pylint: disable=redefined-builtin
    HTTPError, RequestException, ConnectionError
)

from onapsdk.exceptions import (
    RequestError, APIError, ResourceNotFound, InvalidResponse,
    ConnectionFailed
)

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)


class OnapService(ABC):
    """
    Mother Class of all ONAP services.

    An important attribute when inheriting from this class is `_jinja_env`.
    it allows to fetch simply jinja templates where they are.
    by default jinja engine will look for templates in `templates` directory of
    the package.
    See in Examples to see how to use.

    Attributes:
        server (str): nickname of the server we send the request. Used in logs
            strings. For example, 'SDC' is the nickame for SDC server.
        headers (Dict[str, str]): the headers dictionnary to use.
        proxy (Dict[str, str]): the proxy configuration if needed.
        permanent_headers (Optional[Dict[str, str]]): optional dictionary of
            headers which could be set by the user and which are **always**
            added into sended request. Unlike the `headers`, which could be
            overrided on `send_message` call these headers are constant.

    """

    @dataclass
    class PermanentHeadersCollection:
        """Collection to store permanent headers."""

        ph_dict: Dict[str, Any] = field(default_factory=dict)
        ph_call: List[Callable] = field(default_factory=list)

        def __iter__(self) -> Iterator[Dict[str, any]]:
            """Iterate through the headers.

            For dictionary based headers just return the dict and
            for the callables iterate through the list of them,
            call them and yield the result.
            """
            yield self.ph_dict
            for ph_call in self.ph_call:
                yield ph_call()

    _logger: logging.Logger = logging.getLogger(__qualname__)
    server: str = None
    headers: Dict[str, str] = {
        "Content-Type": "application/json",
        "Accept": "application/json",
    }
    proxy: Dict[str, str] = None
    permanent_headers: PermanentHeadersCollection = PermanentHeadersCollection()

    def __init_subclass__(cls):
        """Subclass initialization.

        Add _logger property for any OnapService with it's class name as a logger name
        """
        super().__init_subclass__()
        cls._logger: logging.Logger = logging.getLogger(cls.__qualname__)

    def __init__(self) -> None:
        """Initialize the service."""

    @classmethod
    def send_message(cls, method: str, action: str, url: str,  # pylint: disable=too-many-locals
                     **kwargs) -> Union[requests.Response, None]:
        """
        Send a message to an ONAP service.

        Args:
            method (str): which method to use (GET, POST, PUT, PATCH, ...)
            action (str): what action are we doing, used in logs strings.
            url (str): the url to use
            exception (Exception, optional): if an error occurs, raise the
                exception given instead of RequestError
            **kwargs: Arbitrary keyword arguments. any arguments used by
                requests can be used here.

        Raises:
            RequestError: if other exceptions weren't caught or didn't raise,
                            or if there was an ambiguous exception by a request
            ResourceNotFound: 404 returned
            APIError: returned an error code within 400 and 599, except 404
            ConnectionFailed: connection can't be established

        Returns:
            the request response if OK

        """
        cert = kwargs.pop('cert', None)
        basic_auth: Dict[str, str] = kwargs.pop('basic_auth', None)
        exception = kwargs.pop('exception', None)
        headers = kwargs.pop('headers', cls.headers).copy()
        if OnapService.permanent_headers:
            for header in OnapService.permanent_headers:
                headers.update(header)
        data = kwargs.get('data', None)
        try:
            # build the request with the requested method
            session = cls.__requests_retry_session()
            if cert:
                session.cert = cert
            OnapService._set_basic_auth_if_needed(basic_auth, session)

            cls._logger.debug("[%s][%s] sent header: %s", cls.server, action,
                              headers)
            cls._logger.debug("[%s][%s] url used: %s", cls.server, action, url)
            cls._logger.debug("[%s][%s] data sent: %s", cls.server, action,
                              data)

            response = session.request(method,
                                       url,
                                       headers=headers,
                                       verify=False,
                                       proxies=cls.proxy,
                                       **kwargs)

            cls._logger.info(
                "[%s][%s] response code: %s",
                cls.server, action,
                response.status_code if response is not None else "n/a")
            cls._logger.debug(
                "[%s][%s] response: %s",
                cls.server, action,
                response.text if (response is not None and
                                  response.headers.get("Content-Type", "") == \
                                      "application/json") else "n/a")

            response.raise_for_status()
            return response

        except HTTPError as cause:
            cls._logger.error("[%s][%s] API returned and error: %s",
                              cls.server, action, headers)

            msg = f'Code: {cause.response.status_code}. Info: {cause.response.text}.'

            if cause.response.status_code == 404:
                exc = ResourceNotFound(msg)
            else:
                exc = APIError(msg)

            raise exc from cause

        except ConnectionError as cause:
            cls._logger.error("[%s][%s] Failed to connect: %s", cls.server,
                              action, cause)

            msg = f"Can't connect to {url}."
            raise ConnectionFailed(msg) from cause

        except RequestException as cause:
            cls._logger.error("[%s][%s] Request failed: %s",
                              cls.server, action, cause)

        if not exception:
            msg = f"Ambiguous error while requesting {url}."
            raise RequestError(msg)

        raise exception

    @classmethod
    def _set_basic_auth_if_needed(cls, basic_auth, session):
        if basic_auth:
            session.auth = (basic_auth.get('username'),
                            basic_auth.get('password'))

    @classmethod
    def send_message_json(cls, method: str, action: str, url: str,
                          **kwargs) -> Dict[Any, Any]:
        """
        Send a message to an ONAP service and parse the response as JSON.

        Args:
            method (str): which method to use (GET, POST, PUT, PATCH, ...)
            action (str): what action are we doing, used in logs strings.
            url (str): the url to use
            exception (Exception, optional): if an error occurs, raise the
                exception given
            **kwargs: Arbitrary keyword arguments. any arguments used by
                requests can be used here.

        Raises:
            InvalidResponse: if JSON coudn't be decoded
            RequestError: if other exceptions weren't caught or didn't raise
            APIError/ResourceNotFound: send_message() got an HTTP error code
            ConnectionFailed: connection can't be established
            RequestError: send_message() raised an ambiguous exception


        Returns:
            the response body in dict format if OK

        """
        exception = kwargs.get('exception', None)
        try:

            response = cls.send_message(method, action, url, **kwargs)

            if response:
                return response.json()

        except simplejson.errors.JSONDecodeError as cause:
            cls._logger.error("[%s][%s]Failed to decode JSON: %s", cls.server,
                              action, cause)
            raise InvalidResponse from cause

        except RequestError as exc:
            cls._logger.error("[%s][%s] request failed: %s",
                              cls.server, action, exc)
            if not exception:
                exception = exc

        raise exception

    @staticmethod
    def __requests_retry_session(retries: int = 10,
                                 backoff_factor: float = 0.3,
                                 session: requests.Session = None
                                 ) -> requests.Session:
        """
        Create a request Session with retries.

        Args:
            retries (int, optional): number of retries. Defaults to 10.
            backoff_factor (float, optional): backoff_factor. Defaults to 0.3.
            session (requests.Session, optional): an existing session to
                enhance. Defaults to None.

        Returns:
            requests.Session: the session with retries set

        """
        session = session or requests.Session()
        retry = Retry(
            total=retries,
            read=retries,
            connect=retries,
            backoff_factor=backoff_factor,
        )
        adapter = HTTPAdapter(max_retries=retry)
        session.mount('http://', adapter)
        session.mount('https://', adapter)
        return session

    @staticmethod
    def set_proxy(proxy: Dict[str, str]) -> None:
        """
        Set the proxy for Onap Services rest calls.

        Args:
            proxy (Dict[str, str]): the proxy configuration

        Examples:
            >>> OnapService.set_proxy({
            ...     'http': 'socks5h://127.0.0.1:8082',
            ...     'https': 'socks5h://127.0.0.1:8082'})

        """
        OnapService.proxy = proxy

    @staticmethod
    def set_header(header: Optional[Union[Dict[str, Any], Callable]] = None) -> None:
        """Set the header which will be always send on request.

        The header can be:
            * dictionary - will be used same dictionary for each request
            * callable - a method which is going to be called every time on request
              creation. Could be useful if you need to connect with ONAP through some API
              gateway and you need to take care about authentication. The callable shouldn't
              require any parameters
            * None - reset headers

        Args:
            header (Optional[Union[Dict[str, Any], Callable]]): header to set. Defaults to None

        """
        if not header:
            OnapService._logger.debug("Reset headers")
            OnapService.permanent_headers = OnapService.PermanentHeadersCollection()
            return
        if callable(header):
            OnapService.permanent_headers.ph_call.append(header)
        else:
            OnapService.permanent_headers.ph_dict.update(header)
        OnapService._logger.debug("Set permanent header %s", header)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# SPDX-License-Identifier: Apache-2.0
"""Clamp module."""
from onapsdk.configuration import settings
from onapsdk.onap_service import OnapService as Onap
from onapsdk.sdc.service import Service
from onapsdk.exceptions import ResourceNotFound
from onapsdk.utils.headers_creator import headers_clamp_creator


class Clamp(Onap):
    """Mother Class of all CLAMP elements."""

    #class variable
    _base_url = settings.CLAMP_URL
    name: str = "CLAMP"
    headers = headers_clamp_creator(Onap.headers)

    @classmethod
    def base_url(cls) -> str:
        """Give back the base url of Clamp."""
        return f"{cls._base_url}/restservices/clds/v2"

    @classmethod
    def check_loop_template(cls, service: Service) -> str:
        """
        Return loop template name if exists.

        Args:
            service (Service): the distributed sdc service with tca blueprint artifact

        Raises:
            ResourceNotFound: Template not found.

        Returns:
            if required template exists in CLAMP or not

        """
        url = f"{cls.base_url()}/templates/"
        for template in cls.send_message_json('GET',
                                              'Get Loop Templates',
                                              url):
            if template["modelService"]["serviceDetails"]["name"] == service.name:
                return template["name"]
        raise ResourceNotFound("Template not found.")

    @classmethod
    def check_policies(cls, policy_name: str, req_policies: int = 30) -> bool:
        """
        Ensure that a policy is stored in CLAMP.

        Args:
            policy_name (str): policy acronym
            req_policies (int): number of required policies in CLAMP

        Returns:
            if required policy exists in CLAMP or not

        """
        url = f"{cls.base_url()}/policyToscaModels/"
        policies = cls.send_message_json('GET',
                                         'Get stocked policies',
                                         url)
        exist_policy = False
        for policy in policies:
            if policy["policyAcronym"] == policy_name:
                exist_policy = True
        return (len(policies) >= req_policies) and exist_policy



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# SPDX-License-Identifier: Apache-2.0
"""SDNC base module."""
from onapsdk.configuration import settings
from onapsdk.onap_service import OnapService


class SdncElement(OnapService):
    """SDNC base class."""

    base_url = settings.SDNC_URL



# -*- coding: utf-8 -*-
# SPDX-License-Identifier: Apache-2.0
"""SDC category management module."""

import json
from abc import ABC, abstractmethod
from typing import Any, Dict, List

from onapsdk.configuration import settings
from onapsdk.exceptions import ResourceNotFound
from onapsdk.sdc import SDC
from onapsdk.utils.headers_creator import headers_sdc_generic


class BaseCategory(SDC, ABC):  # pylint: disable=too-many-instance-attributes
    """Base SDC category class.

    It's SDC admin resource, has no common properties with
        SDC resourcer or elements, so SDC class can't be it's
        base class.

    """

    SDC_ADMIN_USER = "demo"

    def __init__(self, name: str) -> None:
        """Service category initialization.

        Args:
            name (str): Service category name.

        """
        super().__init__(name)
        self.normalized_name: str = None
        self.unique_id: str = None
        self.icons: List[str] = None
        self.subcategories: List[Dict[str, str]] = None
        self.version: str = None
        self.owner_id: str = None
        self.empty: bool = None
        self.type: str = None

    @classmethod
    def _get_all_url(cls) -> str:
        """Get URL for all categories in SDC."""
        return f"{cls.base_front_url}/sdc1/feProxy/rest/v1/setup/ui"

    @classmethod
    def _base_url(cls) -> str:
        """Give back the base url of Sdc."""
        return f"{settings.SDC_FE_URL}/sdc1/feProxy/rest/v1/category"

    @classmethod
    def headers(cls) -> Dict[str, str]:
        """Headers used for category management.

        It uses SDC admin user.

        Returns:
            Dict[str, str]: Headers

        """
        return headers_sdc_generic(super().headers, user=cls.SDC_ADMIN_USER)

    @classmethod
    def get_all(cls, **kwargs) -> List['SDC']:
        """
        Get the categories list created in SDC.

        Returns:
            the list of the categories

        """
        return super().get_all(headers=cls.headers())

    @classmethod
    def import_from_sdc(cls, values: Dict[str, Any]) -> 'BaseCategory':
        """
        Import category object from SDC.

        Args:
            values (Dict[str, Any]): dict to parse returned from SDC.

        """
        category_obj = cls(name=values["name"])
        category_obj.normalized_name = values["normalizedName"]
        category_obj.unique_id = values["uniqueId"]
        category_obj.icons = values["icons"]
        category_obj.subcategories = values["subcategories"]
        category_obj.version = values["version"]
        category_obj.owner_id = values["ownerId"]
        category_obj.empty = values["empty"]
        return category_obj

    @classmethod
    @abstractmethod
    def category_name(cls) -> str:
        """Class category name.

        Used for logs.

        Returns:
            str: Category name

        """

    @classmethod
    def get(cls, name: str) -> "BaseCategory":
        """Get category with given name.

        Raises:
            ResourceNotFound: Category with given name does not exist

        Returns:
            BaseCategory: BaseCategory instance

        """
        category_obj: "BaseCategory" = cls(name)
        if category_obj.exists():
            return category_obj
        msg = f"{cls.category_name()} with \"{name}\" name does not exist."
        raise ResourceNotFound(msg)

    @classmethod
    def create(cls, name: str) -> "BaseCategory":
        """Create category instance.

        Checks if category with given name exists and if it already
            exists just returns category with given name.

        Returns:
            BaseCategory: Created category instance

        """
        category_obj: "BaseCategory" = cls(name)
        if category_obj.exists():
            return category_obj
        cls.send_message_json("POST",
                              f"Create {name} {cls.category_name()}",
                              cls._base_create_url(),
                              data=json.dumps({"name": name}),
                              headers=cls.headers())
        category_obj.exists()
        return category_obj

    def _copy_object(self, obj: 'BaseCategory') -> None:
        """
        Copy relevant properties from object.

        Args:
            obj (BaseCategory): the object to "copy"

        """
        self.name = obj.name
        self.normalized_name = obj.normalized_name
        self.unique_id = obj.unique_id
        self.icons = obj.icons
        self.subcategories = obj.subcategories
        self.version = obj.version
        self.owner_id = obj.owner_id
        self.empty = obj.empty


class ResourceCategory(BaseCategory):
    """Resource category class."""

    @classmethod
    def _get_objects_list(cls,
                          result: Dict[str, Any]) -> List[Dict[str, Any]]:
        """
        Get list of resource categories.

        Args:
            result (List[Dict[str, Any]]): the result returned by SDC
                in a list of dicts

        Raises:
            KeyError: Invalid result dictionary

        """
        return result["categories"]["resourceCategories"]

    @classmethod
    def _base_create_url(cls) -> str:
        """Url to create resource category.

        Returns:
            str: Creation url

        """
        return f"{cls._base_url()}/resources"

    @classmethod
    def category_name(cls) -> str:
        """Resource category name.

        Used for logging.

        Returns:
            str: Resource category name

        """
        return "Resource Category"

    @classmethod
    def get(cls, name: str, subcategory: str = None) -> "ResourceCategory":  # pylint: disable=arguments-differ
        """Get resource category with given name.

        It returns resource category with all subcategories by default. You can
            get resource category with only one subcategory if you provide it's
            name as `subcategory` parameter.

        Args:
            name (str): Resource category name.
            subcategory (str, optional): Name of subcategory. Defaults to None.

        Raises:
            ResourceNotFound: Subcategory with given name does not exist

        Returns:
            BaseCategory: BaseCategory instance

        """
        category_obj: "ResourceCategory" = super().get(name=name)
        if not subcategory:
            return category_obj
        filtered_subcategories: Dict[str, str] = list(filter(lambda x: x["name"] == subcategory,
                                                             category_obj.subcategories))
        if not filtered_subcategories:
            raise ResourceNotFound(f"Subcategory {subcategory} does not exist.")
        category_obj.subcategories = filtered_subcategories
        return category_obj


class ServiceCategory(BaseCategory):
    """Service category class."""

    @classmethod
    def _get_objects_list(cls,
                          result: Dict[str, Any]) -> List[Dict[str, Any]]:
        """
        Get list of service categories.

        Args:
            result (List[Dict[str, Any]]): the result returned by SDC
                in a list of dicts

        Raises:
            KeyError: Invalid result dictionary

        """
        return result["categories"]["serviceCategories"]

    @classmethod
    def _base_create_url(cls) -> str:
        """Url to create service category.

        Returns:
            str: Creation url

        """
        return f"{cls._base_url()}/services"

    @classmethod
    def category_name(cls) -> str:
        """Service category name.

        Used for logging.

        Returns:
            str: Service category name

        """
        return "Service Category"
